@@ -193,6 +193,8 @@ class OptionParser(optparse.OptionParser):
                           "colors (never, always or auto)")
         optgrp.add_option("--diff", action="store_true", dest="diff",
                           help="show diff between gathered outputs")
+        optgrp.add_option("--outdir", action="store", dest="outdir", help="output directory for stdout files (OPTIONAL)")
+        optgrp.add_option("--errdir", action="store", dest="errdir", help="output directory for sterr files (OPTIONAL)")
         self.add_option_group(optgrp)
 
     def _copy_callback(self, option, opt_str, value, parser):
@@ -237,6 +239,8 @@ class OptionParser(optparse.OptionParser):
                           dest="command_timeout",
                           help="limit time for command to run on the node",
                           type="float")
+        optgrp.add_option("-m", "--mode", action="store", dest="mode",
+                          help="run mode; define MODEs in <confdir>/*.conf")
         optgrp.add_option("-R", "--worker", action="store", dest="worker",
                           help="worker name to use for command execution "
                                "('exec', 'rsh', 'ssh', etc. default is 'ssh')")
