@@ -186,12 +186,16 @@ class Task(object):
         """Special task control port event handler.
         When a message is received on the port, call appropriate
         task method."""
+        def __init__(self, task):
+            EventHandler.__init__(self)
+            self.task = task
+
         def ev_msg(self, port, msg):
             """Message received: call appropriate task method."""
             # pull out function and its arguments from message
             func, (args, kwargs) = msg[0], msg[1:]
             # call task method
-            func(port.task, *args, **kwargs)
+            func(self.task, *args, **kwargs)
 
     class tasksyncmethod(object):
         """Class encapsulating a function that checks if the calling
@@ -254,7 +258,7 @@ class Task(object):
             self._cond.acquire()
             try:
                 self.suspend_count = min(self.suspend_count, 0)
-                self._cond.notifyAll()
+                self._cond.notify_all()
             finally:
                 self._cond.release()
 
@@ -320,9 +324,8 @@ class Task(object):
             self._reset()
 
             # special engine port for task method dispatching
-            self._dispatch_port = EnginePort(self,
-                                            handler=Task._SyncMsgHandler(),
-                                            autoclose=True)
+            self._dispatch_port = EnginePort(handler=Task._SyncMsgHandler(self),
+                                             autoclose=True)
             self._engine.add(self._dispatch_port)
 
             # set taskid used as Thread name
@@ -347,7 +350,7 @@ class Task(object):
     def _is_task_self(self):
         """Private method used by the library to check if the task is
         task_self(), but do not create any task_self() instance."""
-        return self.thread == threading.currentThread()
+        return self.thread == threading.current_thread()
 
     def default_excepthook(self, exc_type, exc_value, tb):
         """Default excepthook for a newly Task. When an exception is
@@ -421,10 +424,25 @@ class Task(object):
         """
         self.topology = TopologyParser(topology_file).tree(_getshorthostname())
 
-    def _default_router(self):
-        if self.router is None:
-            self.router = PropagationTreeRouter(str(self.topology.root.nodeset),
-                                                self.topology)
+    def _default_router(self, router=None):
+        """
+        Helper to instantiate or bind a default PropagationTreeRouter
+        for the task which can then be shared by multiple workers.
+        Called by a TreeWorker when it is scheduled with this task.
+        """
+        if router is None:
+            if self.router is None:
+                # Init router with the task's topology (e.g. root node)
+                self.router = \
+                    PropagationTreeRouter(str(self.topology.root.nodeset),
+                                          self.topology)
+        else:
+            if self.router is not None:
+                # Update default router if a different one is used by a worker.
+                logger = logging.getLogger(__name__)
+                logger.debug("_default_router: overriding previous default " \
+                             "router %s with %s", self.router, router)
+            self.router = router
         return self.router
 
     def default(self, default_key, def_val=None):
@@ -688,7 +706,7 @@ class Task(object):
         is not set, the task can only receive messages on the port by
         calling port.msg_recv().
         """
-        port = EnginePort(self, handler, autoclose)
+        port = EnginePort(handler, autoclose)
         self._add_port(port)
         return port
 
@@ -765,7 +783,7 @@ class Task(object):
 
     def _resume(self):
         """Resume task - called from self thread."""
-        assert self.thread == threading.currentThread()
+        assert self.thread == threading.current_thread()
         try:
             try:
                 self._reset()
@@ -780,7 +798,7 @@ class Task(object):
             # task becomes joinable
             self._join_cond.acquire()
             self._suspend_cond.atomic_inc()
-            self._join_cond.notifyAll()
+            self._join_cond.notify_all()
             self._join_cond.release()
 
     def resume(self, timeout=None):
@@ -954,14 +972,14 @@ class Task(object):
         # termination (late join()s)
         # must be called after _terminated is set to True
         self._join_cond.acquire()
-        self._join_cond.notifyAll()
+        self._join_cond.notify_all()
         self._join_cond.release()
 
         # destroy task if needed
         if kill:
             Task._task_lock.acquire()
             try:
-                del Task._tasks[threading.currentThread()]
+                del Task._tasks[threading.current_thread()]
             finally:
                 Task._task_lock.release()
 
@@ -1376,7 +1394,7 @@ def task_self(defaults=None):
     provided as a convenience is available in the top-level ClusterShell.Task
     package namespace.
     """
-    return Task(thread=threading.currentThread(), defaults=defaults)
+    return Task(thread=threading.current_thread(), defaults=defaults)
 
 def task_wait():
     """
@@ -1385,7 +1403,7 @@ def task_wait():
     convenience and is available in the top-level ClusterShell.Task package
     namespace.
     """
-    Task.wait(threading.currentThread())
+    Task.wait(threading.current_thread())
 
 def task_terminate():
     """
